import pytest

from ckan.tests import factories

from ckanext.relationship.utils import entity_name_by_id


@pytest.mark.usefixtures("clean_db")
class TestEntityNameById:
    def test_entity_name_by_id_when_package_exists(self):
        dataset = factories.Dataset()
        assert entity_name_by_id(dataset["id"]) == dataset["name"]

    def test_entity_name_by_id_when_organization_exists(self):
        organization = factories.Organization()
        assert entity_name_by_id(organization["id"]) == organization["name"]

    def test_entity_name_by_id_when_group_exists(self):
        group = factories.Group()
        assert entity_name_by_id(group["id"]) == group["name"]

    def test_entity_name_by_id_when_no_entity_exists(self):
        assert entity_name_by_id("nonexistent") is None



from __future__ import annotations

import ckan.plugins.toolkit as tk
from ckan.logic import NotFound

import ckanext.scheming.helpers as sch


def get_relations_info(pkg_type: str) -> list[tuple[str, str, str]]:
    """Return information about relation (related_entity, related_entity_type,
    relation_type) of specified package type (pkg_type) from schema.

    Returns:
        List of tuples of related entities: entity, entity_type, relation_type.
    """
    schema = sch.scheming_get_schema("dataset", pkg_type)
    if not schema:
        return []
    return [
        (
            field["related_entity"],
            field["related_entity_type"],
            field["relation_type"],
        )
        for field in schema["dataset_fields"]
        if "relationship_related_entity" in field.get("validators", "")
    ]


def get_relation_field(
    pkg_type: str,
    object_entity: str,
    object_entity_type: str,
    relation_type: str,
) -> dict[str, str]:
    """Return field dict for specified package type (pkg_type) describes relation
    with specified entity (object_entity, object_entity_type) and type of relation
    (relation_type).
    """
    schema = sch.scheming_get_schema("dataset", pkg_type)
    if not schema:
        return {}
    for field in schema["dataset_fields"]:
        if (
            field.get("related_entity") == object_entity
            and field.get("related_entity_type") == object_entity_type
            and field.get("relation_type") == relation_type
        ):
            return field
    return {}


def entity_name_by_id(entity_id: str) -> str:
    """Retrieves the name of an entity given its ID.
    The entity can be a package, organization, or group.
    """
    actions = ["package_show", "organization_show", "group_show"]

    for action in actions:
        try:
            entity = tk.get_action(action)({"ignore_auth": True}, {"id": entity_id})
            if entity:
                return entity.get("name")
        except NotFound:
            pass
    return None



from __future__ import annotations

from typing import Any

from flask import jsonify
from sqlalchemy import or_

import ckan.plugins.toolkit as tk
from ckan import authz, logic
from ckan.logic import validate

from ckanext.relationship import utils
from ckanext.relationship.config import views_without_relationships_in_package_show
from ckanext.relationship.logic import schema
from ckanext.relationship.model.relationship import Relationship
from ckanext.relationship.utils import entity_name_by_id

NotFound = logic.NotFound


def get_actions():
    return {
        "relationship_relation_create": relationship_relation_create,
        "relationship_relation_delete": relationship_relation_delete,
        "relationship_relations_list": relationship_relations_list,
        "relationship_relations_ids_list": relationship_relations_ids_list,
        "relationship_get_entity_list": relationship_get_entity_list,
        "relationship_autocomplete": relationship_autocomplete,
        "package_show": package_show,
    }


@validate(schema.relation_create)
def relationship_relation_create(context, data_dict) -> list[dict[str, str]]:
    """Create relation with specified type (relation_type) between two entities
    specified by ids (subject_id, object_id). Also create reverse relation.
    """
    tk.check_access("relationship_relation_create", context, data_dict)

    subject_id = data_dict["subject_id"]
    object_id = data_dict["object_id"]
    relation_type = data_dict.get("relation_type")

    if Relationship.by_object_id(subject_id, object_id, relation_type):
        return None

    relation = Relationship(
        subject_id=subject_id,
        object_id=object_id,
        relation_type=relation_type,
    )

    reverse_relation = Relationship(
        subject_id=object_id,
        object_id=subject_id,
        relation_type=Relationship.reverse_relation_type[relation_type],
    )

    context["session"].add(relation)
    context["session"].add(reverse_relation)
    context["session"].commit()

    return [rel.as_dict() for rel in (relation, reverse_relation)]


@validate(schema.relation_delete)
def relationship_relation_delete(context, data_dict) -> list[dict[str, str]]:
    """Delete relation with specified type (relation_type) between two entities
    specified by ids (subject_id, object_id). Also delete reverse relation.
    """
    tk.check_access("relationship_relation_delete", context, data_dict)

    subject_id = data_dict["subject_id"]
    subject_name = entity_name_by_id(data_dict["subject_id"])
    object_id = data_dict["object_id"]
    object_name = entity_name_by_id(data_dict["object_id"])
    relation_type = data_dict.get("relation_type")

    relation = (
        context["session"]
        .query(Relationship)
        .filter(
            or_(
                Relationship.subject_id == subject_id,
                Relationship.subject_id == subject_name,
            ),
            or_(
                Relationship.object_id == object_id,
                Relationship.object_id == object_name,
            ),
        )
    )

    if relation_type:
        relation = relation.filter(Relationship.relation_type == relation_type)

    relation = relation.all()

    reverse_relation = (
        context["session"]
        .query(Relationship)
        .filter(
            or_(
                Relationship.subject_id == object_id,
                Relationship.subject_id == object_name,
            ),
            or_(
                Relationship.object_id == subject_id,
                Relationship.object_id == subject_name,
            ),
        )
    )

    if relation_type:
        reverse_relation = reverse_relation.filter(
            Relationship.relation_type
            == Relationship.reverse_relation_type[relation_type],
        )

    reverse_relation = reverse_relation.all()

    [context["session"].delete(rel) for rel in relation]
    [context["session"].delete(rel) for rel in reverse_relation]
    context["session"].commit()
    return [rel[0].as_dict() for rel in (relation, reverse_relation) if len(rel) > 0]


@validate(schema.relations_list)
def relationship_relations_list(context, data_dict) -> list[dict[str, str]]:
    """Return a list of dictionaries representing the relations of a specified entity
    (object_entity, object_type) related to the specified type of relation
    (relation_type) with an entity specified by its id (subject_id).
    """
    tk.check_access("relationship_relations_list", context, data_dict)

    subject_id = data_dict["subject_id"]
    object_entity = data_dict.get("object_entity")
    object_entity = (
        "group" if object_entity and object_entity == "organization" else object_entity
    )
    object_type = data_dict.get("object_type")
    relation_type = data_dict.get("relation_type")

    relations = Relationship.by_subject_id(
        subject_id,
        object_entity,
        object_type,
        relation_type,
    )
    if not relations:
        return []
    return [rel.as_dict() for rel in relations]


@validate(schema.relations_ids_list)
def relationship_relations_ids_list(context, data_dict) -> list[str]:
    """Return ids list of specified entity (object_entity, object_type) related
    with specified type of relation (relation_type) with entity specified
    by id (subject_id).
    """
    tk.check_access("relationship_relations_ids_list", context, data_dict)

    rel_list = relationship_relations_list(context, data_dict)

    return list({rel["object_id"] for rel in rel_list})


@validate(schema.get_entity_list)
def relationship_get_entity_list(context, data_dict) -> list[str]:
    """Return ids list of specified entity (entity, entity_type)."""
    tk.check_access("relationship_get_entity_list", context, data_dict)

    model = context["model"]

    entity = data_dict["entity"]
    entity = entity if entity != "organization" else "group"
    entity_class = logic.model_name_to_class(model, entity)

    return (
        context["session"]
        .query(entity_class.id, entity_class.name, entity_class.title)
        .filter(entity_class.state != "deleted")
        .filter(entity_class.type == data_dict["entity_type"])
        .all()
    )


@validate(schema.autocomplete)
def relationship_autocomplete(context, data_dict) -> dict[str, Any]:
    fq = f'type:{data_dict["entity_type"]} -id:{data_dict["current_entity_id"]}'

    if data_dict.get("owned_only") and not (
        authz.is_sysadmin(tk.current_user.id) and not data_dict.get("check_sysadmin")
    ):
        fq += f" creator_user_id:{tk.current_user.id}"

    packages = tk.get_action("package_search")(
        {},
        {
            "q": data_dict.get("incomplete", ""),
            "fq": fq,
            "fl": "id, title",
            "rows": 100,
            "include_private": True,
            "sort": "score desc",
        },
    )["results"]

    if data_dict.get("updatable_only"):
        packages = [
            pkg
            for pkg in packages
            if tk.h.check_access("package_update", {"id": pkg["id"]})
        ]

    format_autocomplete_helper = getattr(
        tk.h,
        data_dict.get("format_autocomplete_helper"),
        tk.h.relationship_format_autocomplete,
    )

    return jsonify(format_autocomplete_helper(packages))


@tk.chained_action
@tk.side_effect_free
def package_show(next_, context, data_dict) -> dict[str, Any]:
    result = next_(context, data_dict)

    pkg_id = result["id"]
    pkg_type = result["type"]

    views_without_relationships = views_without_relationships_in_package_show()

    if (
        tk.get_endpoint()[1] in views_without_relationships
        and "with_relationships" not in data_dict
    ):
        relations_info = utils.get_relations_info(pkg_type)
        for (
            related_entity,
            related_entity_type,
            relation_type,
        ) in relations_info:
            field = utils.get_relation_field(
                pkg_type,
                related_entity,
                related_entity_type,
                relation_type,
            )
            result.pop(field["field_name"], None)
        return result

    relations_info = utils.get_relations_info(pkg_type)
    for (
        related_entity,
        related_entity_type,
        relation_type,
    ) in relations_info:
        field = utils.get_relation_field(
            pkg_type,
            related_entity,
            related_entity_type,
            relation_type,
        )
        result[field["field_name"]] = [
            relation["object_id"]
            for relation in tk.get_action("relationship_relations_list")(
                context,
                {
                    "subject_id": pkg_id,
                    "object_entity": related_entity,
                    "object_type": related_entity_type,
                    "relation_type": relation_type,
                },
            )
        ]
    return result



from __future__ import annotations

import json
from typing import Any

import ckan.plugins.toolkit as tk
from ckan import authz


def get_helpers():
    helper_functions = [
        relationship_get_entity_list,
        relationship_get_current_relations_list,
        relationship_get_selected_json,
        relationship_get_choices_for_related_entity_field,
        relationship_format_autocomplete,
    ]
    return {f.__name__: f for f in helper_functions}


def relationship_get_entity_list(entity, entity_type, include_private=True):
    """Return ids list of specified entity (entity, entity_type)."""
    context = {}
    if entity == "package":
        entity_list = tk.get_action("package_search")(
            context,
            {
                "fq": f"type:{entity_type}",
                "rows": 1000,
                "include_private": include_private,
            },
        )
        entity_list = entity_list["results"]
    else:
        entity_list = tk.get_action("relationship_get_entity_list")(
            context,
            {"entity": entity, "entity_type": entity_type},
        )
        entity_list = [
            {"id": id, "name": name, "title": title} for id, name, title in entity_list
        ]
    return entity_list


def relationship_get_current_relations_list(data, field) -> list[str]:
    """Pull existing relations for form_snippet and display_snippet."""
    subject_id = field.get("id")
    subject_name = field.get("name")
    if not subject_id and not subject_name:
        return []
    related_entity = data["related_entity"]
    related_entity_type = data["related_entity_type"]
    relation_type = data["relation_type"]

    current_relation_by_id = []
    current_relation_by_name = []

    if subject_id:
        current_relation_by_id = tk.get_action("relationship_relations_ids_list")(
            {},
            {
                "subject_id": subject_id,
                "object_entity": related_entity,
                "object_type": related_entity_type,
                "relation_type": relation_type,
            },
        )
    if subject_name:
        current_relation_by_name = tk.get_action("relationship_relations_ids_list")(
            {},
            {
                "subject_id": subject_name,
                "object_entity": related_entity,
                "object_type": related_entity_type,
                "relation_type": relation_type,
            },
        )
    return current_relation_by_id + current_relation_by_name


def relationship_get_selected_json(selected_ids: list | None = None) -> str:
    if not selected_ids:
        return json.dumps([])

    selected_pkgs = []
    for pkg_id in selected_ids:
        try:
            pkg_dict = tk.get_action("package_show")({}, {"id": pkg_id})
            selected_pkgs.append({"name": pkg_dict["id"], "title": pkg_dict["title"]})
        except (tk.ObjectNotFound, tk.NotAuthorized):
            continue
    return json.dumps(selected_pkgs)


def relationship_get_choices_for_related_entity_field(
    field: dict[str, Any],
    current_entity_id: str | None,
) -> list[str | None]:
    entities = relationship_get_entity_list(
        field["related_entity"],
        field["related_entity_type"],
    )

    choices = []

    for entity in entities:
        if entity["id"] == current_entity_id:
            continue

        if field.get("updatable_only", False) and not tk.h.check_access(
            field["related_entity"] + "_update",
            {"id": entity["id"]},
        ):
            continue

        if (
            field.get("owned_only", False)
            and not authz.is_sysadmin(tk.current_user.id)
            and field["related_entity"] == "package"
            and tk.current_user.id != entity["creator_user_id"]
        ):
            continue

        choices.append((entity["id"], entity.get("title") or entity.get("name")))

    choices.sort(key=lambda x: x[1])
    return choices


def relationship_format_autocomplete(packages: dict[str, Any]) -> dict[str, Any]:
    return {
        "ResultSet": {
            "Result": [
                {
                    "name": pkg["id"],
                    "title": pkg["title"],
                }
                for pkg in packages
            ],
        },
    }



import ckan.plugins.toolkit as tk
from ckan import plugins
from ckan.common import CKANConfig
from ckan.lib.search import rebuild
from ckan.logic import NotFound

import ckanext.scheming.helpers as sch
from ckanext.relationship import helpers, utils, views
from ckanext.relationship.logic import action, auth, validators


class RelationshipPlugin(plugins.SingletonPlugin):
    plugins.implements(plugins.IConfigurer)
    plugins.implements(plugins.IActions)
    plugins.implements(plugins.IAuthFunctions)
    plugins.implements(plugins.IValidators)
    plugins.implements(plugins.ITemplateHelpers)
    plugins.implements(plugins.IBlueprint)
    plugins.implements(plugins.IPackageController, inherit=True)

    # IConfigurer
    def update_config(self, config_: CKANConfig):
        tk.add_template_directory(config_, "templates")
        tk.add_public_directory(config_, "public")
        tk.add_resource("assets", "relationship")

    # IActions
    def get_actions(self):
        return action.get_actions()

    # IAuthFunctions
    def get_auth_functions(self):
        return auth.get_auth_functions()

    # IValidators
    def get_validators(self):
        return validators.get_validators()

    # ITemplateHelpers
    def get_helpers(self):
        return helpers.get_helpers()

    # IBlueprint
    def get_blueprint(self):
        return views.get_blueprints()

    # IPackageController
    def after_dataset_create(self, context, pkg_dict):
        context = context.copy()
        context.pop("__auth_audit", None)
        return _update_relations(context, pkg_dict)

    def after_dataset_update(self, context, pkg_dict):
        context = context.copy()
        context.pop("__auth_audit", None)
        return _update_relations(context, pkg_dict)

    def after_dataset_delete(self, context, pkg_dict):
        context = context.copy()
        context.pop("__auth_audit", None)

        subject_id = pkg_dict["id"]

        relations_ids_list = tk.get_action("relationship_relations_ids_list")(
            context,
            {"subject_id": subject_id},
        )

        for object_id in relations_ids_list:
            tk.get_action("relationship_relation_delete")(
                context,
                {"subject_id": subject_id, "object_id": object_id},
            )

            try:
                rebuild(object_id)
            except NotFound:
                pass
        rebuild(subject_id)

    def before_dataset_index(self, pkg_dict):
        pkg_id = pkg_dict["id"]
        pkg_type = pkg_dict["type"]
        schema = sch.scheming_get_schema("dataset", pkg_type)
        if not schema:
            return pkg_dict
        relations_info = utils.get_relations_info(pkg_type)
        for (
            related_entity,
            related_entity_type,
            relation_type,
        ) in relations_info:
            relations_ids = tk.get_action("relationship_relations_ids_list")(
                {},
                {
                    "subject_id": pkg_id,
                    "object_entity": related_entity,
                    "object_type": related_entity_type,
                    "relation_type": relation_type,
                },
            )

            if not relations_ids:
                continue
            field = utils.get_relation_field(
                pkg_type,
                related_entity,
                related_entity_type,
                relation_type,
            )
            pkg_dict[f'vocab_{field["field_name"]}'] = relations_ids

            pkg_dict.pop(field["field_name"], None)

        return pkg_dict

    # CKAN < 2.10 hooks
    def after_create(self, context, data_dict):
        return self.after_dataset_create(context, data_dict)

    def after_update(self, context, data_dict):
        return self.after_dataset_update(context, data_dict)

    def after_delete(self, context, data_dict):
        return self.after_dataset_delete(context, data_dict)

    def before_index(self, pkg_dict):
        return self.before_dataset_index(pkg_dict)


if tk.check_ckan_version("2.10"):
    tk.blanket.config_declarations(RelationshipPlugin)


def _update_relations(context, pkg_dict):
    subject_id = pkg_dict["id"]
    add_relations = pkg_dict.get("add_relations", [])
    del_relations = pkg_dict.get("del_relations", [])
    if not add_relations and not del_relations:
        return pkg_dict
    for object_id, relation_type in del_relations + add_relations:
        if (object_id, relation_type) in add_relations:
            tk.get_action("relationship_relation_create")(
                context,
                {
                    "subject_id": subject_id,
                    "object_id": object_id,
                    "relation_type": relation_type,
                },
            )
        else:
            tk.get_action("relationship_relation_delete")(
                context,
                {
                    "subject_id": subject_id,
                    "object_id": object_id,
                    "relation_type": relation_type,
                },
            )

        try:
            rebuild(object_id)
        except NotFound:
            pass
    rebuild(subject_id)
    return pkg_dict



from flask import Blueprint

import ckan.plugins.toolkit as tk


def get_blueprints():
    return [
        relationships,
    ]


relationships = Blueprint("relationships", __name__)


@relationships.route("/api/2/util/relationships/autocomplete")
def relationships_autocomplete():
    request_args = tk.request.args
    return tk.get_action("relationship_autocomplete")(
        {},
        {
            "incomplete": request_args.get("incomplete"),
            "current_entity_id": request_args.get("current_entity_id"),
            "entity_type": request_args.get("entity_type", "dataset"),
            "updatable_only": tk.asbool(request_args.get("updatable_only")),
            "owned_only": tk.asbool(request_args.get("owned_only")),
            "check_sysadmin": tk.asbool(request_args.get("check_sysadmin")),
            "format_autocomplete_helper": request_args.get(
                "format_autocomplete_helper",
            ),
        },
    )



"""Create relationship table.

Revision ID: aca2ff1d3ce4
Revises:
Create Date: 2021-07-02 14:40:37.719003

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "aca2ff1d3ce4"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    op.create_table(
        "relationship_relationship",
        sa.Column("id", sa.Text, primary_key=True),
        sa.Column("subject_id", sa.Text, nullable=False),
        sa.Column("object_id", sa.Text, nullable=False),
        sa.Column("relation_type", sa.Text, nullable=False),
    )


def downgrade():
    op.drop_table("relationship_relationship")



from __future__ import annotations

import ckan.plugins.toolkit as tk

CONFIG_VIEWS_WITHOUT_RELATIONSHIPS = (
    "ckanext.relationship.views_without_relationships_in_package_show"
)
DEFAULT_VIEWS_WITHOUT_RELATIONSHIPS = ["search", "read"]


def views_without_relationships_in_package_show() -> list[str]:
    return tk.aslist(
        tk.config.get(
            CONFIG_VIEWS_WITHOUT_RELATIONSHIPS,
            DEFAULT_VIEWS_WITHOUT_RELATIONSHIPS,
        ),
    )



import pytest

import ckan.plugins.toolkit as tk
from ckan.tests import factories

from ckanext.relationship.model.relationship import Relationship


@pytest.mark.usefixtures("clean_db")
class TestRelationCreate:
    def test_create_new_relation(self):
        subject_dataset = factories.Dataset()
        object_dataset = factories.Dataset()

        subject_id = subject_dataset["id"]
        object_id = object_dataset["id"]
        relation_type = "related_to"

        result = tk.get_action("relationship_relation_create")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_id": object_id,
                "relation_type": relation_type,
            },
        )

        assert result[0]["subject_id"] == subject_id
        assert result[0]["object_id"] == object_id
        assert result[0]["relation_type"] == relation_type

        assert result[1]["subject_id"] == object_id
        assert result[1]["object_id"] == subject_id
        assert result[1]["relation_type"] == relation_type

    def test_does_not_create_duplicate_relation(self):
        subject_dataset = factories.Dataset()
        object_dataset = factories.Dataset()

        subject_id = subject_dataset["id"]
        object_id = object_dataset["id"]
        relation_type = "related_to"

        tk.get_action("relationship_relation_create")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_id": object_id,
                "relation_type": relation_type,
            },
        )

        result = tk.get_action("relationship_relation_create")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_id": object_id,
                "relation_type": relation_type,
            },
        )

        assert result is None

    def test_relation_is_added_to_db(self):
        subject_dataset = factories.Dataset()
        object_dataset = factories.Dataset()

        subject_id = subject_dataset["id"]
        object_id = object_dataset["id"]
        relation_type = "related_to"

        tk.get_action("relationship_relation_create")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_id": object_id,
                "relation_type": relation_type,
            },
        )

        relation_straight = Relationship.by_object_id(
            subject_id,
            object_id,
            relation_type,
        )
        relation_reverse = Relationship.by_object_id(
            object_id,
            subject_id,
            relation_type,
        )

        assert relation_straight.subject_id == subject_id
        assert relation_straight.object_id == object_id
        assert relation_straight.relation_type == relation_type

        assert relation_reverse.subject_id == object_id
        assert relation_reverse.object_id == subject_id
        assert relation_reverse.relation_type == relation_type

    @pytest.mark.parametrize(
        "relation_type",
        [
            "related_to",
            "parent_of",
            "child_of",
        ],
    )
    def test_different_relation_types(self, relation_type):
        subject_dataset = factories.Dataset()
        object_dataset = factories.Dataset()

        subject_id = subject_dataset["id"]
        object_id = object_dataset["id"]

        result = tk.get_action("relationship_relation_create")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_id": object_id,
                "relation_type": relation_type,
            },
        )

        assert result[0]["relation_type"] == relation_type
        assert (
            result[1]["relation_type"]
            == Relationship.reverse_relation_type[relation_type]
        )


@pytest.mark.usefixtures("clean_db")
class TestRelationDelete:
    def test_relation_delete(self):
        subject_dataset = factories.Dataset()
        object_dataset = factories.Dataset()

        subject_id = subject_dataset["id"]
        object_id = object_dataset["id"]
        relation_type = "related_to"

        tk.get_action("relationship_relation_create")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_id": object_id,
                "relation_type": relation_type,
            },
        )

        result = tk.get_action("relationship_relation_delete")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_id": object_id,
                "relation_type": relation_type,
            },
        )

        assert result[0]["subject_id"] == subject_id
        assert result[0]["object_id"] == object_id
        assert result[0]["relation_type"] == relation_type

        assert result[1]["subject_id"] == object_id
        assert result[1]["object_id"] == subject_id
        assert result[1]["relation_type"] == relation_type

    def test_relation_deleted_from_db(self):
        subject_dataset = factories.Dataset()
        object_dataset = factories.Dataset()

        subject_id = subject_dataset["id"]
        object_id = object_dataset["id"]
        relation_type = "related_to"

        tk.get_action("relationship_relation_create")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_id": object_id,
                "relation_type": relation_type,
            },
        )

        tk.get_action("relationship_relation_delete")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_id": object_id,
                "relation_type": relation_type,
            },
        )

        relation_straight = Relationship.by_object_id(
            subject_id,
            object_id,
            relation_type,
        )
        relation_reverse = Relationship.by_object_id(
            object_id,
            subject_id,
            relation_type,
        )

        assert not relation_straight
        assert not relation_reverse

    def test_relation_delete_after_dataset_delete(self):
        subject_dataset = factories.Dataset()
        object_dataset = factories.Dataset()

        subject_id = subject_dataset["id"]
        object_id = object_dataset["id"]
        relation_type = "related_to"

        tk.get_action("relationship_relation_create")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_id": object_id,
                "relation_type": relation_type,
            },
        )

        tk.get_action("package_delete")({"ignore_auth": True}, {"id": subject_id})

        relation_straight = Relationship.by_object_id(
            subject_id,
            object_id,
            relation_type,
        )
        relation_reverse = Relationship.by_object_id(
            object_id,
            subject_id,
            relation_type,
        )

        assert not relation_straight
        assert not relation_reverse


@pytest.mark.usefixtures("clean_db")
class TestRelationList:
    @pytest.mark.parametrize(
        ("subject_factory", "object_factory", "object_entity", "object_type"),
        [
            (factories.Dataset, factories.Dataset, "package", "dataset"),
            (factories.Dataset, factories.Organization, "organization", "organization"),
            (factories.Dataset, factories.Group, "group", "group"),
            (factories.Organization, factories.Dataset, "package", "dataset"),
            (
                factories.Organization,
                factories.Organization,
                "organization",
                "organization",
            ),
            (factories.Organization, factories.Group, "group", "group"),
            (factories.Group, factories.Dataset, "package", "dataset"),
            (factories.Group, factories.Organization, "organization", "organization"),
            (factories.Group, factories.Group, "group", "group"),
        ],
    )
    def test_relation_list(
        self,
        subject_factory,
        object_factory,
        object_entity,
        object_type,
    ):
        subject = subject_factory()
        object = object_factory()

        subject_id = subject["id"]
        object_id = object["id"]
        relation_type = "related_to"

        tk.get_action("relationship_relation_create")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_id": object_id,
                "relation_type": relation_type,
            },
        )

        result = tk.get_action("relationship_relations_list")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_entity": object_entity,
                "object_type": object_type,
                "relation_type": relation_type,
            },
        )

        assert result[0]["subject_id"] == subject_id
        assert result[0]["object_id"] == object_id
        assert result[0]["relation_type"] == relation_type

    def test_relation_list_empty(self):
        subject_dataset = factories.Dataset()

        subject_id = subject_dataset["id"]
        relation_type = "related_to"

        result = tk.get_action("relationship_relations_list")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_entity": "package",
                "object_type": "dataset",
                "relation_type": relation_type,
            },
        )

        assert result == []

    def test_relation_list_after_dataset_delete(self):
        subject_dataset = factories.Dataset()
        object_dataset = factories.Dataset()

        subject_id = subject_dataset["id"]
        object_id = object_dataset["id"]
        relation_type = "related_to"

        tk.get_action("relationship_relation_create")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_id": object_id,
                "relation_type": relation_type,
            },
        )

        tk.get_action("package_delete")({"ignore_auth": True}, {"id": subject_id})

        result = tk.get_action("relationship_relations_list")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_entity": "package",
                "object_type": "dataset",
                "relation_type": relation_type,
            },
        )

        assert result == []


@pytest.mark.usefixtures("clean_db")
class TestRelationsIdsList:
    def test_relations_ids_list(self):
        subject_dataset = factories.Dataset()
        object1_dataset = factories.Dataset()
        object2_dataset = factories.Dataset()

        subject_id = subject_dataset["id"]
        object1_id = object1_dataset["id"]
        object2_id = object2_dataset["id"]
        relation_type = "related_to"

        tk.get_action("relationship_relation_create")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_id": object1_id,
                "relation_type": relation_type,
            },
        )

        tk.get_action("relationship_relation_create")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_id": object2_id,
                "relation_type": relation_type,
            },
        )

        result = tk.get_action("relationship_relations_ids_list")(
            {"ignore_auth": True},
            {
                "subject_id": subject_id,
                "object_entity": "package",
                "object_type": "dataset",
                "relation_type": relation_type,
            },
        )

        assert object1_id in result
        assert object2_id in result


@pytest.mark.usefixtures("clean_db")
def test_keep_relation_after_dataset_patch():
    subject_dataset = factories.Dataset(type="package_with_relationship")
    object_dataset = factories.Dataset(type="package_with_relationship")

    subject_id = subject_dataset["id"]
    object_id = object_dataset["id"]
    relation_type = "related_to"

    tk.get_action("relationship_relation_create")(
        {"ignore_auth": True},
        {
            "subject_id": subject_id,
            "object_id": object_id,
            "relation_type": relation_type,
        },
    )

    tk.get_action("package_patch")(
        {"ignore_auth": True},
        {
            "id": subject_id,
            "title": "New title",
        },
    )

    relation_straight = Relationship.by_object_id(
        subject_id,
        object_id,
        relation_type,
    )

    relation_reverse = Relationship.by_object_id(
        object_id,
        subject_id,
        relation_type,
    )

    assert relation_straight is not None
    assert relation_reverse is not None



from __future__ import annotations

from typing import Any

from ckan import types
from ckan.plugins import toolkit as tk


def get_auth_functions():
    auth_functions = [
        relationship_relation_create,
        relationship_relation_delete,
        relationship_relations_list,
        relationship_relations_ids_list,
        relationship_get_entity_list,
        relationship_relationship_autocomplete,
    ]
    return {f.__name__: f for f in auth_functions}


def relationship_relation_create(context: types.Context, data_dict: dict[str, Any]):
    return {"success": True}


def relationship_relation_delete(context: types.Context, data_dict: dict[str, Any]):
    return {"success": True}


@tk.auth_allow_anonymous_access
def relationship_relations_list(context: types.Context, data_dict: dict[str, Any]):
    return {"success": True}


@tk.auth_allow_anonymous_access
def relationship_relations_ids_list(context: types.Context, data_dict: dict[str, Any]):
    return {"success": True}


@tk.auth_allow_anonymous_access
def relationship_get_entity_list(context: types.Context, data_dict: dict[str, Any]):
    return {"success": True}


@tk.auth_allow_anonymous_access
def relationship_relationship_autocomplete(
    context: types.Context,
    data_dict: dict[str, Any],
):
    return {"success": True}
